/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.util.EventObject;

import javax.swing.tree.DefaultTreeCellEditor;

import ch.cern.lhcb.xmleditor.specific.IconsManager;
import ch.cern.lhcb.xmleditor.specific.XmlDisplay;

/**
 * This extends the DefaultTreeCellEditor to allow or not the edition depending
 * on Xml specific conditions
 * 
 * @author Sebastien Ponce
 */
public class TreeCellEditor extends DefaultTreeCellEditor {

    /**
     * an icon manager to be used by this editor
     */
    private IconsManager iconsManager;

    /**
     * the underlying XmlTree
     */
    private XmlTree jTree;

    /**
     * a error message displayer
     */
    private MessageDisplayer msgDis;

    /**
     * default constructor, calls the ancestor one
     * 
     * @param jTree the XmlTree on which this editor will work
     */
    public TreeCellEditor(XmlTree jTree, MessageDisplayer msgDis) {
        super(jTree, (TreeCellRenderer) jTree.getCellRenderer());
        this.jTree = jTree;
        this.msgDis = msgDis;
        this.iconsManager = jTree.getTreeView().getEditor().getIconsManager();
    }

    /**
     * overrides the ancestor's method to avoid edition of node with no editable
     * name
     * 
     * @param event the EventObject corresponding to the begining edition
     */
    public boolean isCellEditable(EventObject event) {
        boolean result = super.isCellEditable(event);
        // if edition is possible, check the node to edit is editable
        if (result) {
            XmlTreeNode treeNode = (XmlTreeNode) jTree.getPathForRow(
                    this.lastRow).getLastPathComponent();
            // if editable, check that there is something to edit
            if (!treeNode.isNameEditable()) {
                msgDis.popupMessage("Not editable since it has no name");
                msgDis
                        .displayMessage("The node you tried to edit does not display an "
                                + "editable name but the default one, computed "
                                + "from its type.\nThus, you cannot change it.\n");
                return false;
            }
            // if still editable, set the editing icon
            String name = treeNode.getNode().getNodeName();
            // deals with dereferenced references
            if (treeNode.getOldNode() != null && !treeNode.isReference()) {
                name = name + "deref";
            }
            XmlDisplay xmlDisplay = treeNode.getXmlDisplay();
            if (xmlDisplay != null) {
                if (!xmlDisplay.isInDtd(treeNode.getNode().getNodeName())) {
                    if (!name.startsWith("#")) {
                        name = "notindtd";
                    }
                }
            }
            this.editingIcon = iconsManager.getIcon(name);
        }
        return result;
    }

}