/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.transfer;

import java.awt.Cursor;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragSource;
import java.awt.event.MouseEvent;
import java.util.Arrays;
import java.util.Vector;

import javax.swing.tree.TreePath;

import ch.cern.lhcb.xmleditor.gui.XmlTree;
import ch.cern.lhcb.xmleditor.gui.XmlTreeNode;
import ch.cern.lhcb.xmleditor.specific.IconsManager;
import ch.cern.lhcb.xmleditor.undo.CompoundEdit;

/**
 * this is an implementation of the interface DragGestureListener that deals
 * with the drag and drop recognizing in the XmlEditor.
 * 
 * @author Sebastien Ponce
 */
public class DragGestureRecognizer implements java.awt.dnd.DragGestureListener {

    /**
     * the underlying tree
     */
    private XmlTree tree;

    /**
     * the drag source
     */
    private DragSource dragSource;

    /**
     * the underlying source listener
     */
    private DragSourceListener dragSourceListener;

    /**
     * constructor
     * 
     * @param tree the underlying jTree
     * @param dragSource drag source
     */
    public DragGestureRecognizer(XmlTree tree, DragSource dragSource) {
        super();
        this.tree = tree;
        this.dragSource = dragSource;
        // create the source listener
        dragSourceListener = new DragSourceListener(tree);
    }

    /**
     * A DragGestureRecognizer has detected a platform-dependent Drag and Drop
     * action initiating gesture and is notifying this Listener in order for it
     * to initiate the action for the user.
     * 
     * @param dge The DragGestureEvent describing the gesture that has just
     *        occured
     */
    public void dragGestureRecognized(DragGestureEvent dge) {
        // ensures it is a drag and drop :
        // if the event is also recognized as popup menu event, it is not one
        MouseEvent mouseEvent = (MouseEvent) dge.getTriggerEvent();
        if (mouseEvent.isPopupTrigger()) { return; }
        // gets the XmlTreeNodes selected
        TreePath[] treePaths = tree.getSelectionPaths();
        TreePath treePath = tree.getClosestPathForLocation(mouseEvent.getX(),
                mouseEvent.getY());
        // if we are not currently over the selection, gives up
        if (treePaths == null) { return; }
        if (!(Arrays.asList(treePaths).contains(treePath))) { return; }
        Vector nodes = new Vector();
        for (int i = 0; i < treePaths.length; i++) {
            nodes.add(((XmlTreeNode) treePaths[i].getLastPathComponent())
                    .getNode());
        }
        // retrieves the icon of the dragged node
        try {
            // selects the cursor of the drag depending on the user action
            IconsManager iconsManager = tree.getTreeView().getEditor()
                    .getIconsManager();
            Cursor cursor = null;
            switch (dge.getDragAction()) {
            case DnDConstants.ACTION_MOVE:
                cursor = iconsManager.getMoveDropCursor(true);
                break;
            case DnDConstants.ACTION_COPY:
                cursor = iconsManager.getCopyDropCursor(true);
                break;
            case DnDConstants.ACTION_LINK:
                cursor = iconsManager.getLinkDropCursor(true);
                break;
            }
            dragSource.startDrag(dge, cursor, new TransferableNode(nodes),
                    dragSourceListener);
            // begins a compoundedit for undo management of the dnd action
            tree.getTreeView().getEditor().getUndoManager().addEdit(
                    new CompoundEdit(tree.getTreeView().getEditor()));
        } catch (java.awt.dnd.InvalidDnDOperationException exception) {
            // nothing to do : this should never happend but a bug in the
            // dnd system makes it happend...
        }
    }

}