/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.xerces;

import java.util.HashSet;
import java.util.Iterator;

import javax.swing.tree.TreePath;

import org.w3c.dom.Node;

import ch.cern.lhcb.xmleditor.gui.TreeView;
import ch.cern.lhcb.xmleditor.gui.XmlTree;
import ch.cern.lhcb.xmleditor.gui.XmlTreeNode;
import ch.cern.lhcb.xmleditor.specific.Model;
import ch.cern.lhcb.xmleditor.specific.XmlFile;

/**
 * This class subclasses the default TextImpl implementation to take some
 * displaying facts into account
 * 
 * @author Sebastien Ponce
 */
public class TextImpl extends org.apache.xerces.dom.TextImpl implements
        TreeElement {

    /**
     * this is the set of TreeNodes associated with this element
     */
    private HashSet treeNodes;

    /**
     * constructor
     * 
     * @param ownerDoc the document that created it
     * @param data the content of the new Text node
     */
    public TextImpl(DocumentImpl ownerDoc, String data) {
        super(ownerDoc, data);
        treeNodes = new HashSet();
    }

    /**
     * Returns a duplicate of this node.
     * 
     * @param deep if true, performs a deep copy
     */
    public Node cloneNode(boolean deep) {
        TextImpl result = (TextImpl) super.cloneNode(deep);
        result.treeNodes = new HashSet();
        return result;
    }

    /**
     * adds treeNode to the list of treeNode associated with this node
     * 
     * @param treeNode the TreeNode to add to the list
     */
    public void addTreeNode(XmlTreeNode treeNode) {
        treeNodes.add(treeNode);
    }

    /**
     * removes treeNode from the list of treeNode associated with this node
     * 
     * @param treeNode the TreeNode to remove from the list
     */
    public void removeTreeNode(XmlTreeNode treeNode) {
        treeNodes.remove(treeNode);
    }

    /**
     * updates the upToDate flag of the file corresponding to this node
     */
    private void updateFile() {
        XmlFile file = ((DocumentImpl) this.getOwnerDocument()).getXmlFile();
        if (file != null) {
            file.upToDate = false;
        }
    }

    /**
     * this method should be called whenever the name of this node is edited
     */
    public void nodeNameChanged() {
        // simply calls nodeValueChanged since the name of a text node is its
        // value
        this.nodeValueChanged();
    }

    /**
     * this method should be called whenever the value of this node is edited
     */
    public void nodeValueChanged() {
        // updates the upToDate field of the corresponding file
        this.updateFile();
        // updates the different displays of this node
        Iterator iterator = treeNodes.iterator();
        while (iterator.hasNext()) {
            XmlTreeNode treeNode = (XmlTreeNode) iterator.next();
            // redisplays the whole tree to validate the changes
            // treeNode.getXmlTree().getTreeView().redisplayAll();
        }
    }

    /**
     * this method should be called when one representation of this node is
     * removed
     * 
     * @param treeNode the representation to be removed. No check done on this
     *        parameter. I MUST be non-null and representing this node we are in
     */
    private void internalRepresentationRemoved(XmlTreeNode treeNode) {
        // gets the corresponding tree
        XmlTree tree = treeNode.getXmlTree();
        // gets the corresponding model
        Model model = tree.getXmlModel();
        // notice if the node was selected
        boolean selected = tree
                .isPathSelected(new TreePath(treeNode.getPath()));
        // for some tricky reason, the subelement has to be displayed to be
        // removed so we try to know if it was and do what is needed. This is
        // actually due to the implementation of XmlTreeNode that hides the non
        // displayed nodes. See this class for more details
        TreeView treeView = treeNode.getXmlTree().getTreeView();
        boolean valueDisplayedAsAttribute = treeView
                .isValueDisplayedAsAttribute();
        if (valueDisplayedAsAttribute) {
            // display the values in the tree
            treeView.setValueDisplayedAsAttribute(false);
        }
        // removes the TreeNode from the tree
        model.removeNodeFromParent(treeNode);
        if (valueDisplayedAsAttribute) {
            // display the values as attribute as it was the case
            treeView.setValueDisplayedAsAttribute(true);
        }
        // removes treeNode from the list of representations for this node
        this.removeTreeNode(treeNode);
        // if the node was selected, re-initializes the editor Pane
        if (selected) {
            tree.setSelectionPath(null);
            tree.getTreeView().eraseEditorPane();
        }
        // redisplays the whole tree to validate the changes
        tree.getTreeView().redisplayAll();
    }

    /**
     * this method should be called when this node is removed
     */
    public void removed() {
        // updates the upToDate field of the corresponding file
        this.updateFile();
        // for each representation of the parent
        while (!treeNodes.isEmpty()) {
            // gets the treeNode
            XmlTreeNode treeNode = (XmlTreeNode) treeNodes.iterator().next();
            // removes it
            internalRepresentationRemoved(treeNode);
        }
    }

    /**
     * gets the value of this node, if it has one. The value of such a node is
     * in fact the value of its text child if it exists
     * 
     * @return the value of this node, null it has none
     */
    public String getValue() {
        return this.getNodeValue();
    }

    /**
     * sets the value of this node, if it has one. The value of such a node is
     * in fact the value of its text child if it exists
     * 
     * @param value the new value of this node
     */
    public void setValue(String value) {
        this.setNodeValue(value);
    }

    /**
     * says whether node is an ancestor of this node or not Take care that the
     * notion of ancestor does include the parent-child relation as well as the
     * element-attribute one. This means that an element is ancestor of its
     * attributes.
     * 
     * @param node the node to be tested
     * @return true if the current node is a descendant of node
     */
    public boolean isNodeAncestor(TreeElement node) {
        if (node == this) { return true; }
        TreeElement parent = (TreeElement) this.getParentNode();
        if (parent == null) {
            return false;
        } else {
            return parent.isNodeAncestor(node);
        }
    }
}
