/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

/**
 * Maps <code>DataStorage</code> content to table representation
 * <p>
 * This class does not contain any data itself, this is just a mapper
 */
public class DataTableModel extends javax.swing.table.AbstractTableModel
        implements javax.swing.event.TreeModelListener {

    private DataStorage storage;

    /**
     * Creates new DataTableModel
     * 
     * @param s storage that contains actual data
     */
    public DataTableModel(DataStorage s) {
        storage = s;
        storage.addTreeModelListener(this);
    }

    /**
     * Returns number of columns
     * 
     * @return columns number
     */
    public int getColumnCount() {
        int k = 0;
        for (int i = 0; i < storage.getGroupsSize(); i++) {
            k += storage.getChannelsSize(i);
        }
        return k + 1;
    }

    /**
     * Returns column class
     * 
     * @param column column index
     * @return column class
     */
    public Class getColumnClass(int column) {
        if (column == 0) return Integer.class;

        return getValueAt(0, column).getClass();
    }

    /**
     * Returns column name
     * 
     * @param column column index
     * @return column name
     */
    public java.lang.String getColumnName(int column) {
        int k = 0;
        if (column == 0) return "";

        for (int i = 0; i < storage.getGroupsSize(); i++) {
            for (int j = 0; j < storage.getChannelsSize(i); j++) {
                k++;
                if (k == column) return storage.getChannel(i, j).getName();
            }
        }
        return "null";
    }

    /**
     * Returns number of rows
     * 
     * @return rows number
     */
    public int getRowCount() {
        int k = 0;
        for (int i = 0; i < storage.getGroupsSize(); i++) {
            for (int j = 0; j < storage.getChannelsSize(i); j++) {
                DataChannel channel = storage.getChannel(i, j);
                k = Math.max(k, channel.size());
            }
        }
        return k;
    }

    /**
     * Sets object for the cell
     * 
     * @param row row index
     * @param column column index
     * @param value object value
     */
    public void setValueAt(Object value, int row, int column) {
        if (column == 0) return;
        int k = 0;

        for (int i = 0; i < storage.getGroupsSize(); i++) {
            for (int j = 0; j < storage.getChannelsSize(i); j++) {
                k++;
                if (k == column) {
                    DataChannel channel = storage.getChannel(i, j);
                    if (row < channel.size()) {
                        channel.setData(row, ((Double) value).doubleValue());
                        storage.updateModel(i);
                    }
                }
            }
        }
    }

    /**
     * Returns object for the cell
     * 
     * @param row row index
     * @param column column index
     * @return object value
     */
    public java.lang.Object getValueAt(int row, int column) {
        int k = 0;
        if (column == 0) return new Integer(row + 1);

        for (int i = 0; i < storage.getGroupsSize(); i++) {
            for (int j = 0; j < storage.getChannelsSize(i); j++) {
                k++;
                if (k == column) {
                    DataChannel channel = storage.getChannel(i, j);
                    if (row < channel.size())
                        return new Double(channel.getData(row));
                }
            }
        }
        return "--";
    }

    /**
     * Is this cell editable?
     * 
     * @param row row index
     * @param column column index
     * @return true if editable
     */
    public boolean isCellEditable(int row, int column) {
        if (column == 0) return false;
        return true;
    }

    /**
     * @param evt
     */
    public void treeStructureChanged(javax.swing.event.TreeModelEvent evt) {
    }

    /**
     * @param evt
     */
    public void treeNodesInserted(javax.swing.event.TreeModelEvent evt) {
        if (evt.getTreePath().getPathCount() > 1) {
            fireTableStructureChanged();
        }
    }

    /**
     * @param evt
     */
    public void treeNodesRemoved(javax.swing.event.TreeModelEvent evt) {
        if (evt.getTreePath().getPathCount() > 1) {
            fireTableStructureChanged();
        }
    }

    /**
     * @param evt
     */
    public void treeNodesChanged(javax.swing.event.TreeModelEvent evt) {
    	fireTableStructureChanged();
    }

}